<?php
// local changes.

global $mfp_required_fields;

require_once 'lib/location.php';

$mfp_required_fields = array(
	'headline', 'about_me', 'gender', 'seeking_gender',
	'seeking_relationship', 'state_nation',

	// special ones

	// check if age > 18
	'birthday',

	// if state_nation contains 'U.S.'
	'zip_code'
);

elgg_register_event_handler('init', 'system', 'mfp_local_init');

function mfp_local_init() {

	// stupid ie8 fix for the hover menu.
	elgg_register_simplecache_view('css/ie8');

	// hook into profile updates to geocode users.
	elgg_register_event_handler('profileupdate','user', 'mfp_local_profile_udpate');

	// draw a custom index page
	elgg_register_page_handler('', 'mfp_local_custom_index');

	// customize default icons
	elgg_register_plugin_hook_handler('entity:icon:url', 'user', 'mfp_local_user_avatar_hook');
	elgg_register_plugin_hook_handler('head', 'page', 'mfp_local_page_head_hook');

	// check values when registering and saving profile info.
	// the edit profile check also clears the imported validation status if it's set.
	elgg_register_plugin_hook_handler('action', 'register', 'mfp_local_check_profile_info_register', 1);
	elgg_register_plugin_hook_handler('action', 'profile/edit', 'mfp_local_check_profile_info_edit', 1);

	// don't let unvalidated users login.
	elgg_register_event_handler('login:before', 'user', 'mfp_local_check_login');

	// set the forward for loggin in users
	elgg_register_plugin_hook_handler('action', 'login', 'mfp_local_login_redirector');

	// have a page for validating migrated users.
	elgg_register_page_handler('tgpersonals', 'mfp_local_tgpersonals_validate');

	// hook into the profile route to record views
	elgg_register_plugin_hook_handler('route', 'profile', 'mfp_local_profile_visit');

	// add "visitors" widget
	elgg_register_widget_type('mfp_local_profile_visitors',
			elgg_echo('mfp_local:widget:profile_visitors'),
			elgg_echo('mfp_local:widget:profile_visitors:desc'),
			array('dashboard'));

	// need to use our own create user handler to pull in profile manager's stuff
	// because default access is wrong.
	elgg_unregister_event_handler('create', 'user', 'profile_manager_create_user_event');
	elgg_register_event_handler('create', 'user', 'mfp_local_profile_manager_create_user');

	// add a recently added tab
	$user = elgg_get_logged_in_user_entity();
	$recently_added = array(
		'text' => elgg_echo('tidypics:mostrecent'),
		'href' => (elgg_instanceof($user, 'user')) ? "photos/recently_added/$user->name" : "photos/recently_added/",
		'name' => 'recently_added',
		'priority' => 600,
		'context' => 'photos'
	);

	elgg_register_menu_item('filter', $recently_added);

	// serve 403 errors
	elgg_register_plugin_hook_handler('forward', 403, 'elgg_error_page_handler');


	// catch the "delete and report as spammer" action to delete all messages from the user.
	elgg_register_plugin_hook_handler('action', 'spam_login_filter/delete', 'mfp_delete_sent_messages');

	// add user hover entry to delete all sent message from user.
	elgg_register_action('delete_sent_messages', dirname(__FILE__) . '/actions/delete_sent_messages.php', 'admin');
	if (elgg_is_admin_logged_in()) {
		elgg_register_plugin_hook_handler('register', 'menu:user_hover', 'mfp_delete_sent_messages_menu');
	}
	
	// change the text on the reported content link
	elgg_register_plugin_hook_handler('register', 'menu:extras', 'mfp_extras_menu');
	
	elgg_register_admin_menu_item('administer', 'avatarrestore', 'administer_utilities');
	elgg_register_action('mfp/avatarrestore', __DIR__ . '/actions/mfp/avatarrestore.php', 'admin');
}

/**
 * Serves the user's avatar
 *
 * @param string $hook
 * @param string $entity_type
 * @param string $returnvalue
 * @param array  $params
 * @return string
 * @access private
 */
function mfp_local_user_avatar_hook($hook, $entity_type, $returnvalue, $params) {
	$user = $params['entity'];
	$size = $params['size'];

	if (isset($user->icontime)) {
		// Try and get the icon
		$filehandler = new ElggFile();
		$filehandler->owner_guid = $user->guid;
		$filehandler->setFilename("profile/{$user->guid}{$size}.jpg");
		
		if (file_exists($filehandler->getFilenameOnFilestore())) {
			return "avatar/view/$user->username/$size/$user->icontime";
		}
		else {
			$ia = elgg_set_ignore_access(true);
			$user->icontime = null;
			elgg_set_ignore_access($ia);
			return "mod/mfp_local/graphics/default{$size}.gif";
		}
	} else {
		return "mod/mfp_local/graphics/default{$size}.gif";
	}
}

/**
 * Geo code users who enter valid zip codes.
 *
 * @param unknown_type $event
 * @param unknown_type $type
 * @param unknown_type $user
 */
function mfp_local_profile_udpate($event, $type, $user) {
	$zip_code = get_input('zip_code');

	if ($zip_code) {
		$lat_lon = mfp_get_lat_long_from_geocode_cache($zip_code);

		if ($lat_lon) {
			mfplocal_geocode_user($user, $lat_lon['latitude'], $lat_lon['longitude']);
		}
	}
}

/**
 * Geocodes a user with lat and lon into the special mfp_users_geocode table.
 *
 * @param ElggUser $user The user entity to geocode.
 * @param string $latitude The latitude
 * @param string $longitude The longitude
 *
 * @return bool
 */
function mfplocal_geocode_user($user, $latitude, $longitude, $access_id = ACCESS_PUBLIC) {
	if (!($user instanceof ElggUser)) {
		return false;
	}

	$latitude = sanitise_string($latitude);
	$longitude = sanitise_string($longitude);
	$access_id = sanitise_int($access_id);

	// use empty because 0 is a valid lat / lon
	if (empty($latitude) || empty($longitude)) {
		return false;
	}

	$db_prefix = get_config('dbprefix');

	$q = "INSERT into {$db_prefix}users_geocode
		(guid, latitude, longitude, access_id) values
		($user->guid, '$latitude', '$longitude', '$access_id') on duplicate key
			UPDATE latitude='$latitude', longitude='$longitude', access_id='$access_id'";

	// because IDs can be 0.
	return (false !== insert_data($q));
}

/**
 * Returns an array of latitude and longitude for a user.
 *
 * @param ElggUser $user
 * @return array|false
 */
function mfplocal_get_user_lat_long($user) {
	if (!($user instanceof ElggUser)) {
		return false;
	}

	$db_prefix = get_config('dbprefix');

	$q = "SELECT * FROM {$db_prefix}users_geocode WHERE guid = $user->guid";
	$data = get_data($q);

	if ($data) {
		return array(
			'latitude' => $data[0]->latitude,
			'longitude' => $data[0]->longitude,
		);
	}

	return false;
}

/**
 * Page handler for validation links
 *
 * @param unknown_type $page
 */
function mfp_local_tgpersonals_validate($page) {
	$user = $code = false;

	if (isset($page[0])) {
		$user = get_entity($page[0]);
	}

	if (isset($page[1])) {
		$code = $page[1];
	}

	if (!($user && $code)) {
		register_error(elgg_echo('mfp:validate:cannot_validate'));
		forward(REFERER);
		return false;
	}

	if (mfp_local_validate_migrated_user($user, $code)) {
		login($user);
		system_message(elgg_echo('mfp:imported_user'));
		$url = "profile/$user->username/edit";
		forward($url);
		return true;
	} else {
		register_error(elgg_echo('mfp:validate:cannot_validate'));

		return false;
	}

	return true;
}

/**
 * Marks the user valid if the code matches
 *
 * @param ElggUser $user
 * @param unknown_type $code
 */
function mfp_local_validate_migrated_user(ElggUser $user, $code) {
	$real_code = uservalidationbyemail_generate_code($user->guid, $user->email);
	if ($code == $real_code) {
		return set_user_validation_status($user->guid, true, 'tg_personals_imported');
	}

	return false;
}

/**
 * Sends the validation email for migrated users
 *
 * Reuse the code part of the user validation by email plugin
 *
 * @param Elgguser $user
 */
function mfp_local_request_validation(Elgguser $user) {

	$site = elgg_get_site_entity();

	// Work out validate link
	$code = uservalidationbyemail_generate_code($user->guid, $user->email);
	$link = elgg_normalize_url("tgpersonals/$user->guid/$code");

	// Send validation email
	$subject = elgg_echo('mfp:validate:subject');
	$body = elgg_echo('mfp:validate:body', array($link));

	$result = notify_user($user->guid, $site->guid, $subject, $body, NULL, 'email');

	return $result;
}

/**
 * Don't allow unvalidated users to log in.
 *
 * Set the last forward from to the dashboard if there isn't one set.
 *
 * @param unknown_type $event
 * @param unknown_type $type
 * @param unknown_type $user
 */
function mfp_local_check_login($event, $type, $user) {
	if ($user->validated_method == 'tg_personals_imported_pending') {
		register_error(elgg_echo('mfp:unvalidated_user'));
		mfp_local_request_validation($user);
		return false;
	}

	return null;
}

/*
 * Redirects new logins to the dashboard.
 */
function mfp_local_login_redirector() {
	if (!isset($_SESSION['last_forward_from'])) {
		$_SESSION['last_forward_from'] = '/dashboard';
	}
}

/**
 * Check the info during registration.
 */
function mfp_local_check_profile_info_register() {
	$fields = mfp_local_get_profile_field_values_from_input('register');

	// this handles forwarding.
	mfp_check_fields($fields);

	return null;
}

/**
 * Check info during edit
 */
function mfp_local_check_profile_info_edit() {
	$fields = mfp_local_get_profile_field_values_from_input('edit');

	// this forwards if needed
	mfp_check_fields($fields);

	// force entries to be access_logged_in or access_public
	$simple_access_id = get_input('simple_access_control');

	if ($simple_access_id != ACCESS_PUBLIC && $simple_access_id != ACCESS_LOGGED_IN) {
		set_input('simple_access_control', ACCESS_PUBLIC);
	}

	return null;
}

/**
 * Helper to get all required fields from input.
 *
 * @param string $type
 */
function mfp_local_get_profile_field_values_from_input($type = 'edit') {
	global $mfp_required_fields;

	switch ($type) {
		case 'edit':
			$prepend = '';
			break;

		case 'register':
			$prepend = 'custom_profile_fields_';
			break;

		default:
			return false;
	}

	$fields = array();

	foreach ($mfp_required_fields as $field) {
		$fields[$field] = get_input($prepend . $field, false);
	}

	return $fields;
}


/**
 * Checks fields from edit or register actions
 *
 * @param array $fields Normalized fields
 * @param bool  $quiet  Display errors?
 */
function mfp_check_fields($values = array(), $quiet = false) {
	global $mfp_required_fields;

	$errors = false;

	elgg_make_sticky_form('register');
	elgg_make_sticky_form('profile_manager_register');

	foreach ($mfp_required_fields as $name) {
		$value = isset($values[$name]) ? $values[$name] : false;

		$error_msg = false;

		switch ($name) {
			case 'zip_code':
				// only for the us
				if (!isset($values['state_nation'])) {
					// will be dealt with by default.
					continue;
				}

				if (!strstr($values['state_nation'], 'U.S.')) {
					// don't care about non-us codes.
					continue;
				}

				// check for 12345 and 12345-1234
				// if we're logged in
				if (strstr($value, '-')) {
					$search = "/^[0-9]{5}-[0-9]{4}$/";
				} else {
					$search = "/^[0-9]{5}$/";
				}
				if (!preg_match($search, $value)) {
					$error_msg = elgg_echo('mfp:error:invalid_zip_code');
				}
				break;

			case 'birthday':
				$age = mfp_get_age_from_time($value);
				if ($age < 18) {
					$error_msg = elgg_echo('mfp:error:underage');
				}
				break;

			default:
				if (!$value) {
					$errors = true;
					$error_msg = elgg_echo("mfp:error:$name");
				}
				break;
		}

		if ($error_msg) {
			$errors = true;

			if (!$quiet) {
				register_error($error_msg);
			}
		}

	}

	// don't allow action to continue
	if ($errors) {
		forward(REFERRER);
		return false;
	}

	return true;
}

/**
 * Returns an age from a time or timestamp.
 *
 * @param mixed $time The time as a epoch time stamp or a strtotime() string.
 * @param mixed $now  The time to compare against
 */
function mfp_get_age_from_time($time, $now = null) {
	// don't check for falsey values in case someone was born on the epoch.
	if ($time === null || $time === false) {
		return elgg_echo('mfp:unknown_age');
	}

	if (!is_numeric($time)) {
		$time = strtotime($time);
	}

	if ($now) {
		if (!is_numeric($now)) {
			$now = strtotime($now);
		}
	} else {
		$now = time();
	}

	$age = ($time < 0) ? ($now + ($time * -1)) : ($now - $time);
	// close enough.
	$year = 60 * 60 * 24 * 365.25;
	return floor($age / $year);

}

/**
 * The custom index page.
 */
function mfp_local_custom_index() {
	if (elgg_is_logged_in()) {
		$file = __DIR__ . "/pages/index_logged_in.php";
	} else {
		$file = __DIR__ . "/pages/index_logged_out.php";
	}

	if (!include_once($file)) {
		return false;
	}

	return true;
}

/**
 * Count the visitors to the profile page.
 *
 * @param unknown_type $page
 */
function mfp_local_profile_visit($hook, $type, $params, $return) {

	// wall garden for logged out users.
	if (elgg_get_plugin_setting('profile_walled_garden')) {
		gatekeeper();
	}
	
	$segments = elgg_extract('segments', $params);
	// don't register a view for edit pages.
	if (count($segments) > 1) {
		return $return;
	}

	$logged_in_user = elgg_get_logged_in_user_entity();
	if (!$logged_in_user || $logged_in_user->isAdmin()) {
		return $return;
	}

	$profile_owner = get_user_by_username($segments[0]);
	if (!$profile_owner || $logged_in_user == $profile_owner) {
		return $return;
	}

	mfp_local_touch_entity_relationship($logged_in_user->guid, 'profile_visit', $profile_owner->guid);

	return $return;
}

/**
 * Add or update the timestamp for a relationship.
 *
 * @param unknown_type $guid1
 * @param unknown_type $relationship
 * @param unknown_type $guid2
 */
function mfp_local_touch_entity_relationship($guid_one, $relationship, $guid_two) {
	$guid_one = (int)$guid_one;
	$relationship = sanitise_string($relationship);
	$guid_two = (int)$guid_two;
	
	remove_entity_relationship($guid_one, $relationship, $guid_two);
	return add_entity_relationship($guid_one, $relationship, $guid_two);
}


/**
 * function to add custom profile fields to user on register
 *
 * Overriden to force access_public on MD for search.
 *
 * @param $event
 * @param $object_type
 * @param $object
 * @return unknown_type
 */
function mfp_local_profile_manager_create_user($event, $object_type, $object){
	$custom_profile_fields = array();

	// retrieve all field that were on the register page
	foreach($_POST as $key => $value){
		if(strpos($key, "custom_profile_fields_") === 0){
			$key = substr($key, 22);
			$custom_profile_fields[$key] = get_input("custom_profile_fields_" . $key);
		}
	}

	if(count($custom_profile_fields) > 0 ){
		$categorized_fields = profile_manager_get_categorized_fields(null, true, true);
		$configured_fields = $categorized_fields['fields'];

		// set ignore access
		$ia = elgg_get_ignore_access();
		elgg_set_ignore_access(true);

		foreach($custom_profile_fields as $shortname => $value){

			// determine if $value should be an array
			if(!is_array($value) && !empty($configured_fields)){
				// only do something if it not is already an array
				foreach($configured_fields as $configured_field){
					if($configured_field->metadata_name == $shortname){
						if($configured_field->metadata_type == "tags" || $configured_field->output_as_tags == "yes"){
							$value = string_to_tag_array($value);
							// no need to continue this foreach
							break;
						}
					}
				}
			}

			// use create_metadata to listen to default access
			if (is_array($value)) {
				$i = 0;
				foreach($value as $interval) {
					$i++;
					if ($i == 1) { $multiple = false; } else { $multiple = true; }
					create_metadata($object->guid, $shortname, $interval, 'text', $object->guid, ACCESS_PUBLIC, $multiple);
				}
			} else {
				create_metadata($object->guid, $shortname, $value, 'text', $object->guid, ACCESS_PUBLIC);
			}
		}

		// restore ignore access
		elgg_set_ignore_access($ia);
	}

	if(isset($_FILES["profile_icon"])){
		add_profile_icon($object);
	}
}



/**
 * Deletes a user's sent messages.
 * @todo - vroom this
 *
 * Also used as a plugin hook handler for "Delete user as spam"
 *
 * @param type $hook
 * @param type $type
 * @param type $value
 * @param type $params
 * @return type
 */
function mfp_delete_sent_messages() {
	$user = get_entity((int) get_input('guid'));

	if (!$user instanceof ElggUser) {
		return;
	}

	$options = array(
		'type' => 'object',
		'subtype' => 'messages',
		'metadata_name' => 'fromId',
		'metadata_value' => $user->getGUID(),
		'limit' => 0
	);

	elgg_unregister_event_handler('all', 'all', 'system_log_listener');
	set_time_limit(0);

	$ia = elgg_set_ignore_access(true);
	$messages = new ElggBatch('elgg_get_entities_from_metadata', $options, null, 50);
	$messages->setIncrementOffset(false);

	$r = true;

	foreach ($messages as $message) {
		if (!$message->delete()) {
			$r = false;
		}
	}

	elgg_set_ignore_access($ia);

	return $r;
}

function mfp_delete_sent_messages_menu($hook, $type, $return, $params) {
	$user = $params['entity'];

	if ($user->guid != elgg_get_logged_in_user_guid()) {
		$item = ElggMenuItem::factory(array(
			'name' => "delete_sent_messages",
			'href' => "action/delete_sent_messages?guid={$user->guid}",
			'text' => 'Delete sent messages',
			'is_action' => true,
			'section' => 'admin',
		));
		$return[] = $item;
	}

	return $return;
}


function mfp_local_page_head_hook($h, $t, $r, $p) {
	foreach ($r['links'] as $key => $link) {
		if ($link['rel'] == 'icon') {
			unset($r['links'][$key]);
		}
	}
	
	if ($r['links']) {
		// reset keys
		$r['links'] = array_values($r['links']);
	}
	
	// fix the favicon
	$r['links'][] = array(
		'rel' => 'icon',
		'href' => elgg_get_site_url() . 'mod/mfp_local/graphics/favicon.ico'
	);
	
	return $r;
}


function mfp_extras_menu($h, $t, $r, $p) {
	if (!is_array($r)) {
		return $r;
	}
	
	foreach ($r as $key => $item) {
		if ($item->getName() == 'report_this') {
			$item->setText(elgg_view_icon('report-this') . ' Report This');
		}
	}
	
	return $r;
}