<?php
/**
 * Replace the entire mess that is jQuery.datepicker with 3 simple 
 * dropdowns for year, month, and day.
 *
 * Javascript is added for on change events that updates a hidden form elements in
 * epoch format
 *
 * @uses string $vars['name']   The name for the (hidden) input.
 * @uses string $vars['value']  The value in any strtotime() parsable format or epoch
 */

$value = elgg_extract('value', $vars);
$name = elgg_extract('name', $vars);

if (!$value && $value !== 0) {
	$value = time();
}

// already epochy
if (!is_numeric($value)) {
	$value = strtotime($value);
}

$value_year = date('Y', $value);
$value_month = date('m', $value);
$value_day = date('d', $value);

$years = array();
$end = date('Y') - 18 - 100;
for ($i = date('Y') - 18; $i > $end; $i--) {
	$years[] = $i;
}

$months = array(
	'01' => 'Jan',
	'02' => 'Feb',
	'03' => 'Mar',
	'04' => 'Apr',
	'05' => 'May',
	'06' => 'Jun',
	'07' => 'Jul',
	'08' => 'Aug',
	'09' => 'Sep',
	'10' => 'Oct',
	'11' => 'Nov',
	'12' => 'Dec'
);

$days = array();
for ($i = 1; $i <= 31; $i++) {
	if ($i < 10) {
		$days[] = "0$i";
	} else {
		$days[] = $i;
	}
}

$year = elgg_view('input/dropdown', array(
	'options' => $years,
	'value' => $value_year,
	'class' => 'mfp-date-year'
));

$month = elgg_view('input/dropdown', array(
	'options_values' => $months,
	'value' => $value_month,
	'class' => 'mfp-date-month'
));

$day = elgg_view('input/dropdown', array(
	'options' => $days,
	'value' => $value_day,
	'class' => 'mfp-date-day'
));

echo <<<___HTML
<span class="mfp-date-picker">
$month $day $year
<input type="hidden" name="{$vars['name']}" value="$value" />
</span>
___HTML;
?>
<script>
$(function() {
	$('.mfp-date-picker select').live('change', function() {
		// update the hidden input
		var $container = $(this).parent();
		var $input = $container.find('input[type=hidden]');
		var $year = $container.find('select.mfp-date-year');
		var $month = $container.find('select.mfp-date-month');
		var $day = $container.find('select.mfp-date-day');
		var date = new Date($year.val(), $month.val() - 1, $day.val());
		
		$input.val(date.getTime() / 1000);
	});
});

</script>